package de.ronnyfriedland.cassandra.api

import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer

import org.apache.cassandra.thrift.Cassandra
import org.apache.cassandra.thrift.Column
import org.apache.cassandra.thrift.ColumnOrSuperColumn
import org.apache.cassandra.thrift.ColumnParent
import org.apache.cassandra.thrift.ColumnPath
import org.apache.cassandra.thrift.ConsistencyLevel
import org.apache.cassandra.thrift.InvalidRequestException;
import org.apache.cassandra.thrift.SlicePredicate
import org.apache.cassandra.thrift.SliceRange
import org.apache.cassandra.thrift.TBinaryProtocol
import org.apache.cassandra.thrift.TimedOutException;
import org.apache.cassandra.thrift.UnavailableException;
import org.apache.thrift.TException;
import org.apache.thrift.transport.TFramedTransport
import org.apache.thrift.transport.TSocket
import org.apache.thrift.transport.TTransport

import de.ronnyfriedland.cassandra.entity.BaseEntity
import de.ronnyfriedland.cassandra.wrapper.EntityWrapper

/**
 * Einfacher Cassandra-Client, welcher Zugriffsmethoden für <br/>
 * <ul> <br/>
 * <li>SELECT</li> <br/>
 * <li>INSERT</li> <br/>
 * <li>DELETE</li> <br/>
 * </ul> <br/>
 * zur Verfügung stellt.
 * 
 * @author Ronny Friedland
 */
class CassandraClient {

    protected final String ENCODING = "UTF-8";

    protected final TTransport transport
    protected final Cassandra.Client client

    /**
     * Erzeugt eine neue CassandraClient-Instanz.
     * @param host Cassandra-Host
     * @param port Cassandra-Port
     * @param keySpace zu nutzender Keyspace
     */
    public CassandraClient(String host, Integer port, String keySpace) {
        transport = new TFramedTransport(new TSocket(host, port))
        client = new Cassandra.Client(new TBinaryProtocol(transport))
        transport.open()
        client.set_keyspace(keySpace)
    }

    /**
     * Schliesst die Verbindung zur Cassandra-DB.
     */
    public void close() {
        transport.flush()
        transport.close()
    }

    /**
     * Einfügen eines neuen Datensatzes
     * @param entity zu speicherndes Entity
     */
    public void insert(BaseEntity entity) throws UnsupportedEncodingException, InvalidRequestException, UnavailableException,
    TimedOutException, TException {
        doInsert(EntityWrapper.fromEntity(entity))
    }

    protected void doInsert(def properties) throws UnsupportedEncodingException, InvalidRequestException, UnavailableException,
    TimedOutException, TException {
        if(!transport.open) {
            throw new RuntimeException("Connection already closed !")
        }
        long timeStamp = System.currentTimeMillis()
        ByteBuffer uuid = ByteBuffer.wrap(properties?.uuid?.getBytes(ENCODING))
        ColumnParent parent = new ColumnParent(properties?.columnFamily)

        properties?.each {
            Column col = new Column(ByteBuffer.wrap("${it.key}".getBytes(ENCODING)), ByteBuffer.wrap("${it.value}"
                    .getBytes(ENCODING)), timeStamp)
            client.insert(uuid, parent, col, ConsistencyLevel.ONE)
        }
    }

    /**
     * Selektieren eines Datensatzes anhand des eindeutigen Schlüssels.
     * @param uuid eindeutige ID
     * @param cf zu nutzende KeyFamily
     * @return BaseEntity
     */
    public BaseEntity select(String uuid, String cf) throws UnsupportedEncodingException, InvalidRequestException, UnavailableException,
    TimedOutException, TException {
        def resultProperties = doSelect(uuid, cf)

        BaseEntity entity = EntityWrapper.toEntity(resultProperties)
        return entity
    }

    protected Map doSelect(String uuid, String cf) throws UnsupportedEncodingException, InvalidRequestException, UnavailableException,
    TimedOutException, TException {
        if(!transport.open) {
            throw new RuntimeException("Connection already closed !")
        }
        ColumnParent parent = new ColumnParent(cf)
        SlicePredicate slicePredicate = new SlicePredicate();
        SliceRange sliceRange = new SliceRange();
        sliceRange.setStart(new byte[0]);
        sliceRange.setFinish(new byte[0]);
        slicePredicate.setSlice_range(sliceRange);

        List<ColumnOrSuperColumn> resultColumns = client.get_slice(ByteBuffer.wrap(uuid?.getBytes(ENCODING)), parent,
                slicePredicate, ConsistencyLevel.ONE);

        def timeStamp = -1;
        def resultProperties = [:]
        resultColumns?.each {
            resultProperties[new String(it?.column?.name, ENCODING)] = new String(it?.column?.value, ENCODING)
            resultProperties["creationTime"] = it?.column?.timestamp
        }
        return resultProperties
    }

    /**
     * Löschen eines Datensatzes.
     * @param entity das zu löschende Entity
     */
    public void remove(BaseEntity entity) throws UnsupportedEncodingException, InvalidRequestException, UnavailableException,
    TimedOutException, TException {
        remove(entity?.uuid, entity?.creationTime, entity?.columnFamily)
    }

    /**
     * Löschen eines Datensatzes.
     * @param uuid eindeutige ID des Datensatzes
     * @param creationTime Erstellungszeitpunkt
     * @param cf zu nutzende KeyFamily
     */
    public void remove(String uuid, Long creationTime, String cf) throws UnsupportedEncodingException, InvalidRequestException, UnavailableException,
    TimedOutException, TException {
        if(!transport.open) {
            throw new RuntimeException("Connection already closed !")
        }
        ColumnPath path = new ColumnPath();
        path.column_family = cf;
        client.remove(ByteBuffer.wrap(uuid.getBytes(ENCODING)), path, creationTime, ConsistencyLevel.ALL);
    }
}
