package de.ronnyfriedland.cassandra.api

import java.security.Key;
import java.security.KeyPair;
import java.security.KeyStore
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.Certificate;

import java.nio.ByteBuffer

import javax.crypto.Cipher
import javax.crypto.spec.SecretKeySpec

import org.apache.thrift.transport.TTransport

import de.ronnyfriedland.cassandra.entity.BaseEntity
import de.ronnyfriedland.cassandra.entity.SecureBaseEntity
import de.ronnyfriedland.cassandra.props.CassandraProperties;
import de.ronnyfriedland.cassandra.sec.SecurityUtils
import de.ronnyfriedland.cassandra.sec.ex.SecurityException;
import de.ronnyfriedland.cassandra.wrapper.EntityWrapper

/**
 * Cassandra-Client, welcher die Daten verschlüsselt, bevor sie in der DB abgelegt werden.
 * Beim Abrufen der Daten werden diese wieder entschlüsselt zurück gegeben.
 * 
 * Die Daten werden symmetrisch verschlüsselt. Der Schlüssel wiederum wird asymmetrisch verschlüsselt in der DB abgelegt.
 * 
 * @see de.ronnyfriedland.cassandra.api.CassandraClient
 * 
 * @author Ronny Friedland
 */
class SecureCassandraClient extends CassandraClient {

    final KeyPair keyPair

    final SecurityUtils securityUtils = new SecurityUtils()


    /**
     * Erzeugt eine neue SecureCassandraClient-Instanz.
     * @param host Cassandra-Host
     * @param port Cassandra-Port
     * @param keySpace zu nutzender Keyspace
     */
    public SecureCassandraClient(String host, Integer port, String keySpace) throws SecurityException {
        super(host, port, keySpace)

        KeyStore keystore = securityUtils.loadKeyStore(CassandraProperties.getKeystoreName(), CassandraProperties.getKeystorePassword())
        PrivateKey privateKey = securityUtils.getKeyByAlias(keystore, CassandraProperties.getKeyAlias(), CassandraProperties.getKeyPassword());
        PublicKey publicKey = securityUtils.getPublicKeyByAlias(keystore, CassandraProperties.getKeyAlias());

        keyPair = new KeyPair(publicKey, privateKey)
    }

    /**
     * {@inheritDoc}
     * @see de.ronnyfriedland.cassandra.api.CassandraClient#insert(de.ronnyfriedland.cassandra.entity.BaseEntity)
     */
    public void insert(SecureBaseEntity entity) {
        def secretKey = new SecretKeySpec(entity.secretKey, SecurityUtils.ALGORITHM_SYMMETRIC)

        def resultProperties = EntityWrapper.fromEntity(entity)
        resultProperties.each { prop ->
            if("secretKey" != prop.key && prop.value instanceof byte[]) {
                prop.value = securityUtils.encrypt(secretKey, prop.value)
            }
        }

        // finally encrypt secret key
        resultProperties.secretKey = new String(securityUtils.encryptSecretKey(keyPair.getPublic(), resultProperties.secretKey))

        super.doInsert(resultProperties)
    }

    /**
     * {@inheritDoc}
     * @see de.ronnyfriedland.cassandra.api.CassandraClient#select(java.lang.String, java.lang.String)
     */
    SecureBaseEntity select(String uuid, String cf) {
        def resultProperties = doSelect(uuid, cf)

        // first decrypt secret key
        def x = securityUtils.decryptSecretKey(keyPair.getPrivate(), resultProperties.secretKey.bytes)

        def secretKey = new SecretKeySpec(x, SecurityUtils.ALGORITHM_SYMMETRIC)

        resultProperties.each { prop ->
            if("secretKey" != prop.key && prop.value instanceof byte[]) {
                prop.value = securityUtils.decrypt(secretKey, prop.value)
            }
        }

        BaseEntity entity = EntityWrapper.toEntity(resultProperties)
        return (SecureBaseEntity)entity
    }
}
