package de.ronnyfriedland.cassandra.sec;

import java.security.KeyPair;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;

import javax.crypto.SecretKey;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.junit.Before;

import de.ronnyfriedland.cassandra.props.CassandraProperties;
import de.ronnyfriedland.cassandra.sec.ex.SecurityException;

/**
 * Testet die Klasse {@link SecurityUtils}.
 * 
 * @author Ronny Friedland
 */
public class SecurityUtilsTest extends TestCase {

    private static final java.util.logging.Logger LOG = java.util.logging.Logger.getLogger(SecurityUtilsTest.class
            .getName());

    private SecurityUtils securityUtils;
    private KeyPair keyPair;
    private SecretKey secretKey;

    /**
     * {@inheritDoc}
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @Override
    @Before
    public void setUp() throws Exception {
        securityUtils = new SecurityUtils();

        KeyStore keystore = securityUtils.loadKeyStore(CassandraProperties.getKeystoreName(),
                CassandraProperties.getKeystorePassword());
        PrivateKey privateKey = securityUtils.getKeyByAlias(keystore, CassandraProperties.getKeyAlias(),
                CassandraProperties.getKeyPassword());
        PublicKey publicKey = securityUtils.getPublicKeyByAlias(keystore, CassandraProperties.getKeyAlias());

        keyPair = new KeyPair(publicKey, privateKey);
        secretKey = securityUtils.getSecretKey();
    }

    /**
     * Testet das asymmetrische Ver- und Entschlüsseln
     */
    public void testEncryptDecrypt() {
        String testData = "test test test";
        byte[] decTestData = null;
        try {
            byte[] encrypted = securityUtils.encrypt(secretKey, testData.getBytes());
            decTestData = securityUtils.decrypt(secretKey, encrypted);
        } catch (SecurityException e) {
            Assert.fail("No exception expected: " + e);
        }
        LOG.info(new String(decTestData));
        Assert.assertEquals(testData, new String(decTestData));
    }

    /**
     * Testet das Ver- und Entschlüsseln des geheimen (symmetrischen) Schlüssels
     */
    public void testEncryptDecryptSecretKey() {
        byte[] decSecretKey = null;
        try {
            byte[] encSecretKey = securityUtils.encryptSecretKey(keyPair.getPublic(), secretKey.getEncoded());
            decSecretKey = securityUtils.decryptSecretKey(keyPair.getPrivate(), encSecretKey);
        } catch (SecurityException e) {
            Assert.fail("No exception expected: " + e);
        }
        LOG.info(new String(decSecretKey));
        Assert.assertEquals(new String(secretKey.getEncoded()), new String(decSecretKey));
    }

    /**
     * Testet den vollständigen Durchlauf:<br/>
     * <ol>
     * <li>Daten symmetrisch verschlüsseln</li>
     * <li>Symmetrischen Schlüsseln asymmetrisch verschlüsseln</li>
     * <li>Symmetrischen Schlüssel asymmetrisch entschlüsseln</li>
     * <li>Daten symmetrisch entschlüsseln</li>
     * <ol>
     */
    public void testEncryptDecryptRoundtrip() {
        String testData = "test test test";
        byte[] decTestData = null;
        byte[] decSecretKey = null;
        try {
            byte[] encTestData = securityUtils.encrypt(secretKey, testData.getBytes());
            byte[] encSecretKey = securityUtils.encryptSecretKey(keyPair.getPublic(), secretKey.getEncoded());
            decSecretKey = securityUtils.decryptSecretKey(keyPair.getPrivate(), encSecretKey);
            decTestData = securityUtils.decrypt(secretKey, encTestData);
        } catch (SecurityException e) {
            e.printStackTrace();
            Assert.fail("No exception expected: " + e);
        }
        Assert.assertEquals(testData, new String(decTestData));
        Assert.assertEquals(new String(secretKey.getEncoded()), new String(decSecretKey));

    }
}
